# Copyright (c) 2010 Outpost Embedded, LLC
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.  You should have
# received a copy of the GNU General Public License along with this program; if
# not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
# Floor, Boston, MA 02110-1301, USA. Any Outpost Embedded trademarks that are
# incorporated in the source code or documentation are not subject to the GNU
# General Public License and may only be used or replicated with the express
# permission of Outpost Embedded, LLC.

class Error(Exception):
    def __str__(self):
        return unicode(self).encode('utf-8', 'replace')


class DeviceError(Error):
    base_msg = u'Error accessing the device'
    msg = None

    def __init__(self, msg):
        super(DeviceError, self).__init__(msg)
        self.msg = msg

    def __unicode__(self):
        return u'%s: %s' % (self.base_msg, self.msg)


class DeviceNotFoundError(DeviceError):
    def __init__(self, extra_msg = None):
        msg = u'The device may no longer be accessible.'
        if extra_msg is not None:
            msg = u'%s: %s' % (msg, extra_msg)
        super(DeviceNotFoundError, self).__init__(msg)


class DeviceBusyError(DeviceError):
    def __init__(self, extra_msg = None):
        msg = (
          u'Device is in use. '
          u'Please close any programs that are accessing the device.'
        )
        if extra_msg is not None:
            msg = u'%s: %s' % (msg, extra_msg)
        super(DeviceBusyError, self).__init__(msg)


class SubprocessError(Error):
    returncode = None
    error_output = None

    def __init__(self, returncode, error_output):
        super(SubprocessError, self).__init__(returncode, error_output)
        self.returncode = returncode
        self.error_output = error_output

    def __unicode__(self):
        return u'Subprocess failed: %s' % self.error_output.strip()


class PrivilegeEscalationError(Error):
    def __unicode__(self):
        return (
          u'Failed to gain required administrative privileges. '
          u'Either run this program as root or install gksu, kdesudo, or pkexec.'
        )
